#include "Font.h"
#include <list>
#include <string>
Font::Font()
{
	texID=shadowID=0;
	srcBlend = GL_SRC_ALPHA;
	dstBlend = GL_ONE_MINUS_SRC_ALPHA;
	scale.x = 1;
	scale.y = 1;
	TextColor[0]=1;
	TextColor[1]=1;
	TextColor[2]=1;
	TextColor[3]=1;
	ShadowColor[0]=0;
	ShadowColor[1]=0;
	ShadowColor[2]=0;
	ShadowColor[3]=1;
	mShadowDrop = Vec2(1,1);
	bShadow = true;
	mAlign = ALIGN_CENTER;
	sine =0;
	bSineWave = false;
	bFixedWidth = false;
	mfFixedWidthSpacing = 1;
}
Font::~Font()
{
	if(texID)
		glDeleteTextures(1,&texID);
}

bool Font::LoadFont(char* name)
{
	if(!GenerateGlyphs(name))
		return false;
	GenerateCoords();
	return true;
}
bool Font::GenerateGlyphs(char* name)
{
	ILuint ImageName;
	ilOriginFunc(IL_ORIGIN_UPPER_LEFT);
	ilGenImages(1, &ImageName);
	ilBindImage(ImageName);
	if(!ilLoadImage(name))
	{
		gLog.OutPut("Failed to load fontset: ");
		gLog.OutPut(name);
		gLog.OutPut("!\n");
		ilDeleteImages(1, &ImageName);
		return false;
	}
	gLog.OutPut("Image Loaded:");
	gLog.OutPut(name);
	gLog.OutPut(".\n");

	unsigned int w=ilGetInteger(IL_IMAGE_WIDTH), h=ilGetInteger(IL_IMAGE_HEIGHT);
	unsigned char* data = ilGetData();

	GenerateOffsets(data,w,h);
	glGenTextures(1,&texID);
	UTIL_GL::BindTexture(GL_TEXTURE_2D,texID);
	UTIL_GL::TextureFilter(GL_NEAREST,GL_NEAREST);
	UTIL_GL::TextureWrap(GL_REPEAT, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,w,h,0,GL_RGBA,GL_UNSIGNED_BYTE,data);
	ilDeleteImages(1, &ImageName);
	shadowID = texID;
	return true;
}
void Font::GenerateOffsets(unsigned char* data, int w, int h)
{
	int x,y,pixcount,ox,oy;
	int width = w/16;
	int height = h/16;
	mFontRes.x = width;
	mFontRes.y = height;
	for(ox=0;ox<w;ox+=width)
	{
		for(oy=0;oy<h;oy+=height)
		{
			pixcount=0;
			for(x=0;x<width;x++)
			{
				bool foundpixel = false;
				for(y=0;y<height;y++)
				{
					if(data[4*((ox+x)+((oy+y)*w))+3]>0)
					{
						foundpixel = true;
						break;
					}
					
				}
				if(foundpixel)
					pixcount++;
			}
			if(pixcount>0)
				pixcount++;
			mGlyphs[(ox/width)+((oy/height)*16)].width = pixcount;
		}
	}
}
void Font::GenerateCoords()
{
	float u=0,v=0;
	int x,y;
	for(x=0;x<16;x++)
	{
		v=0;
		for(y=0;y<16;y++)
		{
			mGlyphs[x+(y*16)].tU = u;
			mGlyphs[x+(y*16)].tV = v;
			v+=(1.0f/16.0f);
		}
		u+=(1.0f/16.0f);
	}
}
void Font::SetShadowImage(unsigned int ID)
{
	shadowID = ID;
}
void Font::SetBlendMode(unsigned int src, unsigned int dst)
{
	srcBlend = src;
	dstBlend = dst;
}
void Font::SetScale(float width, float height)
{
	scale.x = width;
	scale.y = height;
}
void Font::EnableShadow()
{
	bShadow = true;
}
void Font::DisableShadow()
{
	bShadow = false;
}

void Font::SetColor(float r, float g, float b, float a)
{
	TextColor[0]=r;
	TextColor[1]=g;
	TextColor[2]=b;
	TextColor[3]=a;
}
void Font::SetShadowColor(float r, float g, float b, float a)
{
	ShadowColor[0]=r;
	ShadowColor[1]=g;
	ShadowColor[2]=b;
	ShadowColor[3]=a;
}
void Font::SetShadowDrop(Vec2 offset)
{
	mShadowDrop = offset;
}

void Font::SetAlignment(int align)
{
	mAlign = align;
}
float Font::GetStringLength(char* txt)
{
	float length=0;
	int l = (int)strlen(txt),n;
	for(n=0;n<l;n++)
	{
		if(txt[n]<=32)
			length+=(4*scale.x);
		else
			bFixedWidth?
			length+=mFontRes.x*scale.x:
			length+=(mGlyphs[txt[n]].width*scale.x);
	}
	return length;
}
void Font::Print(char* txt, float x, float y)
{
	int LoopCount=1;
	switch(mAlign)
	{
	case ALIGN_CENTER:
		x-=(int)(GetStringLength(txt)*0.5f);
		break;
	case ALIGN_RIGHT:
		x-=(int)GetStringLength(txt);
		break;
	}
	float x2=x,y2=y;
	if(bShadow)
		LoopCount=2;
	int l = (int)strlen(txt),n,i;
	for(i=0;i<LoopCount;i++)
	{
		if(i==0 && bShadow)
		{
			UTIL_GL::BindTexture(GL_TEXTURE_2D,shadowID);
			x+=(mShadowDrop.x*scale.x);
			y+=(mShadowDrop.y*scale.y);
			glColor4fv(ShadowColor);
			UTIL_GL::SetBlend(srcBlend,dstBlend);
		}
		else if(i || !bShadow)
		{
			UTIL_GL::BindTexture(GL_TEXTURE_2D,texID);
			x=x2;
			y=y2;
			glColor4fv(TextColor);
			UTIL_GL::SetBlend(srcBlend,dstBlend);
		}
		float oldY= y;
		glBegin(GL_QUADS);
		for(n=0;n<l;n++)
		{
			if(bSineWave)
			{
				float sinus=sin(x*sine);				
				y=oldY+sin(sinus)*sineMulti;
			}
			if(txt[n]<=32 || mGlyphs[txt[n]].width<3)
			{
				x+=(4.0f*scale.x);
			}
			else
			{
				glTexCoord2f(mGlyphs[txt[n]].tU, mGlyphs[txt[n]].tV);
				glVertex2f(x,y);
				glTexCoord2f(mGlyphs[txt[n]].tU+(1.0f/16.0f), mGlyphs[txt[n]].tV);
				glVertex2f(x+(mFontRes.x*scale.x),y);
				glTexCoord2f(mGlyphs[txt[n]].tU+(1.0f/16.0f), mGlyphs[txt[n]].tV+(1.0f/16.0f));
				glVertex2f(x+(mFontRes.x*scale.x),y+(mFontRes.y*scale.y));
				glTexCoord2f(mGlyphs[txt[n]].tU, mGlyphs[txt[n]].tV+(1.0f/16.0f));
				glVertex2f(x,y+(mFontRes.y*scale.y));
				bFixedWidth?
					x+=(mFontRes.x*mfFixedWidthSpacing)*scale.x + 1:
					x+=(mGlyphs[txt[n]].width*scale.x);
			}
		}
		glEnd();
	}
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
}
void Font::PrintClipped(char* txt, float x, float y, float maxlength)
{
	float length=0;
	int l = (int)strlen(txt),n,size=0, count=0;
	std::list<std::string> stringlist;
	for(n=0;n<l;n++,size++)
	{
		if(txt[n]<=32)
			length+=(4*scale.x);
		else
			bFixedWidth?
			length+=mFontRes.x*scale.x:
			length+=(mGlyphs[txt[n]].width*scale.x);
		if(length>=maxlength)
		{
			length = 0;
			std::string temp;
			temp.assign(txt+count,size);
			size = 0;
			count = n;
			stringlist.push_back(temp);
		}
	}
		stringlist.push_back(txt+count);

	float h = 0;
	std::list<std::string>::iterator IT;
	for(IT=stringlist.begin(); IT!=stringlist.end(); IT++)
	{
		Print((char*)(*IT).c_str(),x,y+h);
		h+=10;
	}
}
void Font::EnableSineWave(float value, float multi)
{
	bSineWave = true;
	sine = value;
	sineMulti = multi;
}
void Font::DisableSineWave()
{
	bSineWave = false;
}
void Font::EnableFixedWidth()
{
	bFixedWidth = true;
}
void Font::DisableFixedWidth()
{
	bFixedWidth = false;
}
void Font::SetFixedWidthSpacing(float value)
{
	mfFixedWidthSpacing = value;
}