#include "Frustum.h"
#include "UTIL_openGL.h"

Frustum::Frustum()
{
}

void Frustum::Setup(float fov, float aspect, float begin, float end)
{
	mFov = fov;
	mAspect = aspect;
	mNear = begin;
	mFar = end;
}

void Frustum::ExtractPlanes(Vec3 pos, Vec3 look, Vec3 up)
{
	UTIL_GL::GL3D::SetPerspective(mFov, mAspect, mNear, mFar);
	glGetFloatv( GL_MODELVIEW_MATRIX, External_proj );
	gluLookAt(pos.x,pos.y,pos.z, look.x,look.y,look.z, up.x,up.y,up.z);
	glGetFloatv( GL_MODELVIEW_MATRIX, External_view );
	/* Get the current PROJECTION matrix from OpenGL */
	glGetFloatv( GL_PROJECTION_MATRIX, proj );
	/* Get the current MODELVIEW matrix from OpenGL */
	glGetFloatv( GL_MODELVIEW_MATRIX, model );

	ExtractPlanes();
}
void Frustum::SetMatrices(float* p, float* v)
{
	for(int n=0; n<16; n++)
	{
		proj[n] = p[n];
		model[n] = v[n];
	}
}
void Frustum::ExtractPlanes()
{
	float t;
	/* Get the current PROJECTION matrix from OpenGL */
	glGetFloatv( GL_PROJECTION_MATRIX, proj );
	/* Get the current MODELVIEW matrix from OpenGL */
	glGetFloatv( GL_MODELVIEW_MATRIX, model );

	/* Combine the two matrices (multiply projection by modelview) */
	clip[ 0] = model[ 0] * proj[ 0] + model[ 1] * proj[ 4] + model[ 2] * proj[ 8] + model[ 3] * proj[12];
	clip[ 1] = model[ 0] * proj[ 1] + model[ 1] * proj[ 5] + model[ 2] * proj[ 9] + model[ 3] * proj[13];
	clip[ 2] = model[ 0] * proj[ 2] + model[ 1] * proj[ 6] + model[ 2] * proj[10] + model[ 3] * proj[14];
	clip[ 3] = model[ 0] * proj[ 3] + model[ 1] * proj[ 7] + model[ 2] * proj[11] + model[ 3] * proj[15];

	clip[ 4] = model[ 4] * proj[ 0] + model[ 5] * proj[ 4] + model[ 6] * proj[ 8] + model[ 7] * proj[12];
	clip[ 5] = model[ 4] * proj[ 1] + model[ 5] * proj[ 5] + model[ 6] * proj[ 9] + model[ 7] * proj[13];
	clip[ 6] = model[ 4] * proj[ 2] + model[ 5] * proj[ 6] + model[ 6] * proj[10] + model[ 7] * proj[14];
	clip[ 7] = model[ 4] * proj[ 3] + model[ 5] * proj[ 7] + model[ 6] * proj[11] + model[ 7] * proj[15];

	clip[ 8] = model[ 8] * proj[ 0] + model[ 9] * proj[ 4] + model[10] * proj[ 8] + model[11] * proj[12];
	clip[ 9] = model[ 8] * proj[ 1] + model[ 9] * proj[ 5] + model[10] * proj[ 9] + model[11] * proj[13];
	clip[10] = model[ 8] * proj[ 2] + model[ 9] * proj[ 6] + model[10] * proj[10] + model[11] * proj[14];
	clip[11] = model[ 8] * proj[ 3] + model[ 9] * proj[ 7] + model[10] * proj[11] + model[11] * proj[15];

	clip[12] = model[12] * proj[ 0] + model[13] * proj[ 4] + model[14] * proj[ 8] + model[15] * proj[12];
	clip[13] = model[12] * proj[ 1] + model[13] * proj[ 5] + model[14] * proj[ 9] + model[15] * proj[13];
	clip[14] = model[12] * proj[ 2] + model[13] * proj[ 6] + model[14] * proj[10] + model[15] * proj[14];
	clip[15] = model[12] * proj[ 3] + model[13] * proj[ 7] + model[14] * proj[11] + model[15] * proj[15];

	/* Extract the numbers for the RIGHT planes */
	planes[0][0] = clip[ 3] - clip[ 0];
	planes[0][1] = clip[ 7] - clip[ 4];
	planes[0][2] = clip[11] - clip[ 8];
	planes[0][3] = clip[15] - clip[12];

	/* Normalize the result */
	t = sqrt( planes[0][0] * planes[0][0] + planes[0][1] * planes[0][1] + planes[0][2] * planes[0][2] );
	planes[0][0] /= t;
	planes[0][1] /= t;
	planes[0][2] /= t;
	planes[0][3] /= t;

	/* Extract the numbers for the LEFT planes */
	planes[1][0] = clip[ 3] + clip[ 0];
	planes[1][1] = clip[ 7] + clip[ 4];
	planes[1][2] = clip[11] + clip[ 8];
	planes[1][3] = clip[15] + clip[12];

	/* Normalize the result */
	t = sqrt( planes[1][0] * planes[1][0] + planes[1][1] * planes[1][1] + planes[1][2] * planes[1][2] );
	planes[1][0] /= t;
	planes[1][1] /= t;
	planes[1][2] /= t;
	planes[1][3] /= t;

	/* Extract the BOTTOM planes */
	planes[2][0] = clip[ 3] + clip[ 1];
	planes[2][1] = clip[ 7] + clip[ 5];
	planes[2][2] = clip[11] + clip[ 9];
	planes[2][3] = clip[15] + clip[13];

	/* Normalize the result */
	t = sqrt( planes[2][0] * planes[2][0] + planes[2][1] * planes[2][1] + planes[2][2] * planes[2][2] );
	planes[2][0] /= t;
	planes[2][1] /= t;
	planes[2][2] /= t;
	planes[2][3] /= t;

	/* Extract the TOP planes */
	planes[3][0] = clip[ 3] - clip[ 1];
	planes[3][1] = clip[ 7] - clip[ 5];
	planes[3][2] = clip[11] - clip[ 9];
	planes[3][3] = clip[15] - clip[13];

	/* Normalize the result */
	t = sqrt( planes[3][0] * planes[3][0] + planes[3][1] * planes[3][1] + planes[3][2] * planes[3][2] );
	planes[3][0] /= t;
	planes[3][1] /= t;
	planes[3][2] /= t;
	planes[3][3] /= t;

	/* Extract the FAR planes */
	planes[4][0] = clip[ 3] - clip[ 2];
	planes[4][1] = clip[ 7] - clip[ 6];
	planes[4][2] = clip[11] - clip[10];
	planes[4][3] = clip[15] - clip[14];

	/* Normalize the result */
	t = sqrt( planes[4][0] * planes[4][0] + planes[4][1] * planes[4][1] + planes[4][2] * planes[4][2] );
	planes[4][0] /= t;
	planes[4][1] /= t;
	planes[4][2] /= t;
	planes[4][3] /= t;

	/* Extract the NEAR planes */
	planes[5][0] = clip[ 3] + clip[ 2];
	planes[5][1] = clip[ 7] + clip[ 6];
	planes[5][2] = clip[11] + clip[10];
	planes[5][3] = clip[15] + clip[14];

	/* Normalize the result */
	t = sqrt( planes[5][0] * planes[5][0] + planes[5][1] * planes[5][1] + planes[5][2] * planes[5][2] );
	planes[5][0] /= t;
	planes[5][1] /= t;
	planes[5][2] /= t;
	planes[5][3] /= t;

}
bool Frustum::BoundsInFrustum(Vec3 mins, Vec3 maxs)
{
	int p;
	for( p	= 0; p < 6;	p++	)
	{
		if( planes[p][0]	* (mins.x) + planes[p][1] * (mins.y) +	planes[p][2] *	(mins.z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (maxs.x) + planes[p][1] * (mins.y) +	planes[p][2] *	(mins.z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (mins.x) + planes[p][1] * (maxs.y) +	planes[p][2] *	(mins.z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (maxs.x) + planes[p][1] * (maxs.y) +	planes[p][2] *	(mins.z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (mins.x) + planes[p][1] * (mins.y) +	planes[p][2] *	(maxs.z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (maxs.x) + planes[p][1] * (mins.y) +	planes[p][2] *	(maxs.z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (mins.x) + planes[p][1] * (maxs.y) +	planes[p][2] *	(maxs.z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (maxs.x) + planes[p][1] * (maxs.y) +	planes[p][2] *	(maxs.z) + planes[p][3] > 0 )
			continue;
		return false;
	}
	return true;
}
bool Frustum::FrustraInFrustum(Vec3 points[9])
{
	int p;
	for( p	= 0; p < 6;	p++	)
	{
		if( planes[p][0]	* (points[0].x) + planes[p][1] * (points[0].y) +	planes[p][2] *	(points[0].z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (points[1].x) + planes[p][1] * (points[1].y) +	planes[p][2] *	(points[1].z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (points[2].x) + planes[p][1] * (points[2].y) +	planes[p][2] *	(points[2].z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (points[3].x) + planes[p][1] * (points[3].y) +	planes[p][2] *	(points[3].z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (points[4].x) + planes[p][1] * (points[4].y) +	planes[p][2] *	(points[4].z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (points[5].x) + planes[p][1] * (points[5].y) +	planes[p][2] *	(points[5].z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (points[6].x) + planes[p][1] * (points[6].y) +	planes[p][2] *	(points[6].z) + planes[p][3] > 0 )
			continue;
		if( planes[p][0]	* (points[7].x) + planes[p][1] * (points[7].y) +	planes[p][2] *	(points[7].z) + planes[p][3] > 0 )
			continue;
		return false;
	}
	return true;

}

