#pragma once
#include "Vector.h"

class Plane
{
public:
	Vec3 N;
	Vec3 P0;//a point on the plane
	double D;

	Plane(){}
	Plane(Vec3 n, double d)
	{
		N = n;
		D = d;
	}
	Plane(const Plane& plane)
	{
		N = plane.N;
		D = plane.D;
		P0 = plane.P0;
	}
	Plane(const Vec3& A, const Vec3& B, const Vec3& C)
	{
		P0 = A;
		Vec3 NA((C-A) | 1.0);
		Vec3 NB((C-B) | 1.0);
		N = (NA ^ NB) | 1.0;
		D = -A % N;
	}
	const Plane& operator = (const Plane& plane)
	{
		N = plane.N;
		D = plane.D;
		return *this;
	}
	const bool operator==(const Plane& plane) const
	{
		return N==plane.N && D==plane.D;
	}
	const bool operator!=(const Plane& plane) const
	{
		return !(*this == plane);
	}
	const bool inline PointOnPlane(const Vec3& P) const
	{
		return DistanceToPlane(P)==0;
	}
	const double inline DistanceToPlane(const Vec3& P) const
	{
		return N % P + D;
	}
	const Vec3 inline RayIntersection(const Vec3& P, const Vec3& R) const
	{
		const double A = N % R;
		if(A==0)
			return P;
		return P - R * (DistanceToPlane(P) / A);
	}
	const Vec3 ClosestPointOnPlane(Vec3 P) const
	{
		double d = DistanceToPlane(P);
		return P - N * d;
	}
};