#pragma once
#include <GL/glew.h>
#include <GL/wglew.h>
#ifdef _WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>
#include <GL/wglext.h>
#include <il/il.h>
#include <il/ilu.h>
#include <il/ilut.h>
#include <string>
#include <sstream>
#include "Log.h"
#include "Vector.h"
#define EXTENSION_SUPPORT_1_1	
//#define gLog /##/
#include <fstream>
using namespace std;
namespace UTIL_GL
{
	void GetGLError();
//================================================================================================//
									/****************
									 ** 2D helpers **
									 ****************/
//================================================================================================//
	namespace GL2D
	{
		void SetOrtho(float w, float h);
		void DrawQuad(float sx, float sy, float ex, float ey);
		void DrawTextureQuad(float sx, float sy, float ex, float ey);//draws texture quad with 0-1 texture coordinates
		void DrawWireBox(float sx, float sy, float ex, float ey);
		Vec2 ProjectPointInWindow(Vec3 in, double proj[16], double model[16]);//project a world coord to window
	}
//================================================================================================//
									/****************
									 ** 3D helpers **
									 ****************/
//================================================================================================//
	namespace GL3D
	{
		void SetPerspective(float Fov, float Aspect, float zNear, float zFar);
		void RenderBillBoard(Vec3 pos, Vec3 rgt, Vec3 up, float size);
//		Vec3 ProjectPointInWorld(Vec3 in,  double proj[16], double model[16]);//project a window coordinate to world
		Vec3 ProjectPointInWorld(Vec2 res, Vec2 in);//project a window coordinate to world
		Vec3 ProjectPointInWorld(Vec2 res, Vec2 in, float& Depth);//project a window coordinate to world
		Vec3 ProjectRayInWorld(Vec2 res, Vec3 in);//project a window coordinate to world
	}
//================================================================================================//
									/************************
								     ** texture enviroment **
									 ************************/
//================================================================================================//
	void TextureWrap(const unsigned int s, const unsigned int t);
	void TextureFilter(const unsigned int min, const unsigned int mag);
//================================================================================================//
									/*******************
									 ** image loading **
									 *******************/
//================================================================================================//
	namespace Image
	{
		//note these functions ignore enviroment and filter settings (uses devil's settings)
		GLuint LoadImage(std::string name);
		GLuint LoadImage1D(std::string name);
		GLuint LoadImage(std::string name, const int min, const int mag, const int wrapx, const int wrapy);
		GLuint LoadImage(std::string name, unsigned int& width, unsigned int& height);
		GLuint LoadImageMipMapped(std::string name);
		unsigned char* LoadImageData(std::string name, unsigned int& width, unsigned int& height);//returns pixeldata - you must free this yourself!
		class ImagePrivate
		{
			static ILuint DevilLoadImage(std::string name);
			friend GLuint LoadImage(std::string name);
			friend GLuint LoadImage1D(std::string name);
			friend GLuint LoadImage(std::string name, const int min, const int mag, const int wrapx, const int wrapy);
			friend GLuint LoadImage(std::string name, unsigned int& width, unsigned int& height);
			friend GLuint LoadImageMipMapped(std::string name);
			friend unsigned char*  LoadImageData(std::string name, unsigned int& width, unsigned int& height, unsigned int& bpp);
		};
	}
//================================================================================================//
									/***************
									 ** shortcuts **
									 ***************/
//================================================================================================//
	void BindTexture(const unsigned int type, unsigned int id);
	void SetBlend(const unsigned int src, const unsigned int dst);
	void AlphaTest(const unsigned int func, float alpha);
	void DepthTest(const unsigned int func);
	void StencilTest(const unsigned int func, int ref, unsigned int mask);
#ifdef EXTENSION_SUPPORT_1_1 
	void BindMultiTexture(const unsigned int channel, unsigned int id);//GL_TEXTURE_2D only 
	void DisableTextureUnit(const unsigned int channel);//GL_TEXTURE_2D only 
#endif 
}

extern unsigned int textureMem;
extern GLfloat gMaxAnisotropy;
extern GLfloat gAnisotropy;

