#pragma once
#include <math.h>
#define PI		(3.14159265359f)
#define DEG2RAD(a)	(PI/180*(a))
#define RAD2DEG(a)	(180/PI*(a))

typedef float scalar;
//typedef double scalar;
#define VEC3TOVEC2(v)	Vec2(v.x,v.y)
#define VEC2TOVEC3(v)	Vec3(v.x,v.y,0)
#define VEC2CPV(v)		cpv(v.x,v.y)
#define CPV2VEC(v)		Vec2(v.x,v.y)
class Vec2
{
public:
	union
	{
		struct
		{
			scalar x,y;
		};
		scalar v[2];
	};
	Vec2(scalar a = 0, scalar b = 0) : x(a), y(b) {}
	Vec2(const Vec2 &vec) : x(vec.x), y(vec.y) {}
	Vec2(float* flvec) : x(flvec[0]), y(flvec[1]) {}
	const Vec2 &operator=(const Vec2& vec)
	{
		this->x=vec.x;
		this->y=vec.y;
		return *this;
	}
	const bool operator==(Vec2 vec) const
	{
		if(x!=vec.x || y!=vec.y)
			return false;
		return true;
	}
	const bool operator<(Vec2 vec) const
	{
		if(x>=vec.x && y>=vec.y)
			return false;
		return true;
	}
	const bool operator<=(Vec2 vec) const
	{
		if(x>vec.x && y>vec.y)
			return false;
		return true;
	}
	const bool operator>(Vec2 vec) const
	{
		if(x<=vec.x && y<=vec.y)
			return false;
		return true;
	}
	const bool operator>=(Vec2 vec) const
	{
		if(x<vec.x && y<vec.y)
			return false;
		return true;
	}
	const bool operator!=(Vec2 vec) const
	{ return !(*this==vec); }
	const Vec2 operator+(const Vec2& vec) const
	{ return Vec2(x+vec.x, y+vec.y); }
	const Vec2 operator-(const Vec2& vec) const
	{ return Vec2(x-vec.x, y-vec.y); }
	const Vec2 &operator+=(const Vec2& vec)
	{
		x+=vec.x; y+=vec.y;
		return *this;
	}
	const Vec2 &operator-=(const Vec2& vec)
	{
		x-=vec.x; y-=vec.y;
		return *this;
	}
	const Vec2 operator/(const Vec2& vec) const
	{ return Vec2(x/vec.x, y/vec.y); }

//	const Vec2 operator-() const
//	{ return Vec2(-x,-y); }
	inline const Vec2 operator*(const scalar s) const
	{
		return Vec2(x*s,y*s);
	}
	inline const Vec2 &operator*=(const scalar s)
	{
		x*=s;y*=s;
		return (*this);
	}
	friend inline const Vec2 operator*(const scalar& s, Vec2 vec)
	{
		return vec*s;
	}
	inline const Vec2 operator*(const Vec2& vec)const
	{ return Vec2(x*vec.x,y*vec.y); }
	inline const Vec2 operator/(const scalar s)const
	{ return Vec2(x/s,y/s); }
	inline Vec2 operator/=(const scalar s)
	{
		x/=s;y/=s;
		return (*this);
	}
	friend inline const Vec2 operator/(const scalar& s, Vec2& vec)
	{
		vec.x/=s;vec.y/=s;
		return vec;
	}
	inline const scalar operator%(Vec2& vec)const//dot product
	{ return x*vec.x+y*vec.y; }
	inline scalar Dot(Vec2& vec)const
	{ return (*this)%vec; }
	inline const scalar operator^(Vec2& vec)const//cross product
	{ return x*vec.y-y*vec.x; }
	inline scalar Cross(Vec2& vec)
	{ return (*this)^vec; }
	inline scalar Length()
	{ return scalar(sqrt(x*x+y*y)); }
	inline scalar Distance(const Vec2& vec)
	{ 
		Vec2 dir(x-vec.x,y-vec.y);
		return dir.Length();
	}
	inline const Vec2 Normalize()
	{ return *this /= Length(); }
	inline void MakeVectors(Vec2& forward, Vec2& right)
	{
		scalar cosYaw = (scalar)cos(DEG2RAD(x));
		scalar sinYaw = (scalar)sin(DEG2RAD(x));
		forward = Vec2(cosYaw,sinYaw);
		right = Vec2(scalar(cos(DEG2RAD(x + 90.0))),scalar(sin(DEG2RAD(x + 90.0))));
		forward.Normalize();
		right.Normalize();		
	}
	inline scalar Angle(Vec2 vec)
	{
		return (scalar)atan2(vec.y - y, vec.x - x) * 180 / PI;
	}
};
inline scalar Dot(Vec2 a, Vec2 b)
{
	return a%b;
}
inline scalar Cross(Vec2 a, Vec2 b)
{ return a^b; }
inline const Vec2 Normalize(Vec2 v)
{ return v / v.Length(); }
inline const scalar Distance(Vec2 a, Vec2 b)
{
	Vec2 dir(a-b);
	return dir.Length();
}
inline scalar Length(Vec2 v)
{
	return scalar(sqrt(v.x*v.x+v.y*v.y));
}

inline scalar Angle(Vec2 a, Vec2 b)
{ return (scalar)atan2(b.y - a.y, b.x - a.x) * 180 / PI; }
inline Vec2 MakeNormalForPoints(Vec2 s, Vec2 e)
{
	Vec2 d=s-e;
	Vec2 n = Vec2(-d.y,d.x);
	return Normalize(n);
}
inline Vec2 OrthoProj(Vec2 U, Vec2 V)
{
	return (Dot(U,V) / Dot(U,U)) * U;
}
inline Vec2 Reflect(Vec2 v, Vec2 n, scalar r)
{
	v -= (1.0f + r) * Dot(v,n) * n;
	return  v;
}
/************************************************************************************************
	Vec3
*************************************************************************************************/

class Vec3
{
public:
	union
	{
		struct
		{
			scalar x,y,z;
		};
		scalar v[3];
	};

	Vec3(scalar a = 0, scalar b = 0, scalar c = 0) : x(a), y(b), z(c) {}
	Vec3(const Vec3 &vec) : x(vec.x), y(vec.y), z(vec.z) {}
	Vec3(float* flvec) : x(flvec[0]), y(flvec[1]), z(flvec[2]) {}
	scalar &operator[](const long idx)
	{
		return *((&x)+idx);
	}

	const Vec3 &operator=(const Vec3& vec)
	{
		this->x=vec.x;
		this->y=vec.y;
		this->z=vec.z;
		return *this;
	}
	const Vec3 &operator=(const float* flvec)
	{
		x = flvec[0];
		y = flvec[1];
		z = flvec[2];

		return *this;
	}
	const bool operator==(Vec3 vec) const
	{
		if(x!=vec.x || y!=vec.y || z!=vec.z)
			return false;
		return true;
	}
	const bool operator!=(Vec3 vec) const
	{ return !(*this==vec); }
	const Vec3 operator+(const Vec3& vec) const
	{ return Vec3(x+vec.x, y+vec.y, z+vec.z); }
	const Vec3 operator-(const Vec3& vec) const
	{ return Vec3(x-vec.x, y-vec.y, z-vec.z); }
	const Vec3 &operator+=(const Vec3& vec)
	{
		x+=vec.x; y+=vec.y; z+=vec.z;
		return *this;
	}
	const Vec3 &operator-=(const Vec3& vec)
	{
		x-=vec.x; y-=vec.y; z-=vec.z;
		return *this;
	}
	const Vec3 operator-() const
	{ return Vec3(-x,-y,-z); }
	inline const Vec3 operator*(const scalar s) const
	{ return Vec3(x*s,y*s,z*s); }
	inline const Vec3 &operator*=(const scalar s)
	{
		x*=s;y*=s;z*=s;
		return (*this);
	}
	friend inline const Vec3 operator*(const scalar& s, Vec3& vec)
	{
		vec.x*=s;vec.y*=s;vec.z*=s;
		return vec;
	}
	inline const Vec3 operator*(const Vec3& vec)const
	{ return Vec3(x*vec.x,y*vec.y,z*vec.z); }
	inline const Vec3 operator/(const scalar s)const
	{ return Vec3(x/s,y/s,z/s); }
	inline Vec3 operator/=(const scalar s)
	{
		x/=s;y/=s;z/=s;
		return (*this);
	}
	friend inline const Vec3 operator/(const scalar& s, Vec3& vec)
	{
		vec.x/=s;vec.y/=s;vec.z/=s;
		return vec;
	}
	const scalar operator%(const Vec3& vec)const
	{ return x*vec.x+y*vec.y+z*vec.z; }
	scalar Dot(Vec3& vec)const
	{ return (*this)%vec; }
	const Vec3 Cross(const Vec3 &vec) const
	{ return Vec3(y*vec.z - z*vec.y, z*vec.x - x*vec.z, x*vec.y - y*vec.x); }
	const Vec3 operator^(const Vec3 &vec) const
	{ return Vec3(y*vec.z - z*vec.y, z*vec.x - x*vec.z, x*vec.y - y*vec.x); }
	inline scalar Length()
	{ return scalar(sqrt(x*x+y*y+z*z)); }
	inline const Vec3 Normalize()
	{ return *this /= Length(); }
	inline const Vec3 operator | (const scalar length) const
	{ return *this * (length / sqrtf(x*x+y*y+z*z)); }
	inline scalar Distance(const Vec3& vec)
	{ 
		Vec3 dir(vec-(*this));
		return dir.Length();
	}
	void MakeVectors(Vec3& forward, Vec3& right, Vec3& up)
	{
		scalar cosYaw = (scalar)cos(DEG2RAD(x));
		scalar sinYaw = (scalar)sin(DEG2RAD(x));
		scalar tanPitch = (scalar)tan(DEG2RAD(y));
		forward = Vec3(cosYaw,tanPitch,sinYaw);
		right = Vec3(-sinYaw,0,cosYaw);
		up = forward.Cross(right);

		//something wrong with roll
/*		scalar ct = -cos(DEG2RAD(y));
		scalar st = -sin(DEG2RAD(y));
		scalar cp = cos(DEG2RAD(x));
		scalar sp = sin(DEG2RAD(x));
		scalar cr = cos(DEG2RAD(z));
		scalar sr = sin(DEG2RAD(z));
        right = Vec3(cp*cr,  -sr*cp,   sp);
        up = Vec3(st*sp*cr+ct*sr,  ct*cr-sr*st*sp,  -cp*st);
		forward = right.Cross(up);
*/
		forward.Normalize();
		right.Normalize();
		up.Normalize();
	}
	// return angle between two vectors
	scalar Angle(Vec3 vec)
	{
		return atan2(vec.z - z, vec.x - x) * 180 / PI;
	}
	void AngleBetweenVector(Vec3& angles) const
	{
		float	tmp, yaw, pitch;
		
		if (z == 0 && x == 0)
		{
			yaw = 0;
			if (z > 0)
				pitch = 90;
			else
				pitch = 270;
		}
		else
		{
			yaw = (float)(atan2(z, x) * 180 / PI);
			if (yaw < 0)
				yaw += 360;

			tmp = (float)sqrt (x*x + z*z);
			pitch = (float)(atan2(y, tmp) * 180 / PI);
			if (pitch < 0)
				pitch += 360;
		}
		
		angles.x = pitch;
		angles.z = yaw;
		angles.y = 0;
	}
};
inline scalar Dot(Vec3 a, Vec3 b)
{ return a%b; }
inline Vec3 Cross(Vec3 a, Vec3 b)
{ return a^b; }
inline const Vec3 Normalize(Vec3 v)
{ return v / v.Length(); }
inline const scalar Distance(Vec3 a, Vec3 b)
{
	Vec3 dir(a-b);
	return dir.Length();
}

inline Vec3 MakeNormalForPoints(Vec3 s, Vec3 e)
{
	return Normalize(Cross(s,e));
}
inline Vec3 Reflect(Vec3 v, Vec3 n, scalar r)
{
	v -= (1.0f + r) * Dot(v,n) * n;
	return  v;
}
inline Vec3 Project(Vec3 v, Vec3 n)
{
	return  n * Dot(v,n);
}
//*/
//which side of a line a point is on
inline bool LineTest(Vec2 v1, Vec2 v2, Vec2 v3, Vec2 p)
{
	Vec2 A(v2-v1);
	Vec2 B(p-v1);
	Vec2 cp1(0, A.x*B.y - A.y*B.x);
	Vec2 C(v3-v1);
	Vec2 cp2(0, A.x*C.y - A.y*C.x);
	if(cp1.Dot(cp2) >=0 )
		return true;
	return false;
}
inline bool TrianglePointTest(Vec2 v[3], Vec2 p)
{
	if(!LineTest(v[1],v[2],v[0],p))
		return false;
	if(!LineTest(v[0],v[2],v[1],p))
		return false;
	if(!LineTest(v[0],v[1],v[2],p))
		return false;
	return true;
}
inline bool TriangleBoxTest(Vec2 v[3], Vec2 p, float w, float h)
{
	Vec2 v1 = Vec2(p.x-w,p.y-h);
	Vec2 v2 = Vec2(p.x+w,p.y-h);
	Vec2 v3 = Vec2(p.x+w,p.y+h);
	Vec2 v4 = Vec2(p.x-w,p.y+h);
	
	for(int n=0; n<3; n++)
	{
		if(n==0)
		{
			if(LineTest(v[1],v[2],v[0],v1))
				continue;
			if(LineTest(v[1],v[2],v[0],v2))
				continue;
			if(LineTest(v[1],v[2],v[0],v3))
				continue;
			if(LineTest(v[1],v[2],v[0],v4))
				continue;
			return false;
		}
		else if(n==1)
		{
			if(LineTest(v[0],v[2],v[1],v1))
				continue;
			if(LineTest(v[0],v[2],v[1],v2))
				continue;
			if(LineTest(v[0],v[2],v[1],v3))
				continue;
			if(LineTest(v[0],v[2],v[1],v4))
				continue;
			return false;
		}
		else if(n==2)
		{
			if(LineTest(v[0],v[1],v[2],v1))
				continue;
			if(LineTest(v[0],v[1],v[2],v2))
				continue;
			if(LineTest(v[0],v[1],v[2],v3))
				continue;
			if(LineTest(v[0],v[1],v[2],v4))
				continue;
			return false;
		}
	}
	return true;
}

inline Vec3 RotateAboutPoint(Vec3 in, Vec3 p, scalar x, scalar y, scalar z, scalar d)
{
	Vec3 out;
	x = DEG2RAD(x);
	y = DEG2RAD(y);
	z = DEG2RAD(z);
	float X[3],Y[3],Z[3];
    X[0] = cos(x)*cos(y);
    X[1] = cos(x)*sin(y);
    X[2] = -sin(x);

    Y[0] = sin(z)*sin(x)*cos(y)-cos(z)*sin(y);
    Y[1] = sin(z)*sin(x)*sin(y)+cos(z)*cos(y);
    Y[2] = cos(x)*sin(z);

    Z[0] = cos(z)*sin(x)*cos(y)+sin(z)*sin(y);
    Z[1] = cos(z)*sin(x)*sin(y)-sin(z)*cos(y);
    Z[2] = cos(x)*cos(z);

	Vec3 dir(Normalize(in - p)*d);
    out.x = p.x + dir.x*X[0] + dir.y*X[1] + dir.z*X[2];
    out.y = p.y + dir.x*Y[0] + dir.y*Y[1] + dir.z*Y[2];
    out.z = p.z + dir.x*Z[0] + dir.y*Z[1] + dir.z*Z[2];


	return out;
}
inline void Rotation(Vec3 angles, Vec3& f, Vec3& r, Vec3& u)
{
	float x = DEG2RAD(angles.x);
	float y = DEG2RAD(angles.y);
	float z = DEG2RAD(angles.z);

    f.x = cos(x)*cos(y);
    f.y = cos(x)*sin(y);
    f.z = -sin(x);

    u.x = sin(z)*sin(x)*cos(y)-cos(z)*sin(y);
    u.y = sin(z)*sin(x)*sin(y)+cos(z)*cos(y);
    u.z = cos(x)*sin(z);

    r.x = cos(z)*sin(x)*cos(y)+sin(z)*sin(y);
    r.y = cos(z)*sin(x)*sin(y)-sin(z)*cos(y);
    r.z = cos(x)*cos(z);

}
inline void MatrixInvert(double* mat)
{
	double result[16];
	double tmp[16]	 = {mat[ 0], mat[ 4], mat[ 8], mat[12],
						mat[ 1], mat[ 5], mat[ 9], mat[13],
						mat[ 2], mat[ 6], mat[10], mat[14],
						mat[ 3], mat[ 7], mat[11], mat[15]};
	mat = tmp;
	double det;



	tmp[0] = mat[10] * mat[15];
	tmp[1] = mat[11] * mat[14];
	tmp[2] = mat[9] * mat[15];
	tmp[3] = mat[11] * mat[13];
	tmp[4] = mat[9] * mat[14];
	tmp[5] = mat[10] * mat[13];
	tmp[6] = mat[8] * mat[15];
	tmp[7] = mat[11] * mat[12];
	tmp[8] = mat[8] * mat[14];
	tmp[9] = mat[10] * mat[12];
	tmp[10] = mat[8] * mat[13];
	tmp[11] = mat[9] * mat[12];

	//calculate first 8 elements (cofactors)
	result[0]=		tmp[0]*mat[5] + tmp[3]*mat[6] + tmp[4]*mat[7] -	tmp[1]*mat[5] - tmp[2]*mat[6] - tmp[5]*mat[7];

	result[1]=		tmp[1]*mat[4] + tmp[6]*mat[6] + tmp[9]*mat[7] -	tmp[0]*mat[4] - tmp[7]*mat[6] - tmp[8]*mat[7];

	result[2]=		tmp[2]*mat[4] + tmp[7]*mat[5] + tmp[10]*mat[7] -	tmp[3]*mat[4] - tmp[6]*mat[5] - tmp[11]*mat[7];

	result[3]=		tmp[5]*mat[4] + tmp[8]*mat[5] + tmp[11]*mat[6] -	tmp[4]*mat[4] - tmp[9]*mat[5] - tmp[10]*mat[6];

	result[4]=		tmp[1]*mat[1] + tmp[2]*mat[2] + tmp[5]*mat[3] -	tmp[0]*mat[1] - tmp[3]*mat[2] - tmp[4]*mat[3];

	result[5]=		tmp[0]*mat[0] + tmp[7]*mat[2] + tmp[8]*mat[3] -	tmp[1]*mat[0] - tmp[6]*mat[2] - tmp[9]*mat[3];

	result[6]=		tmp[3]*mat[0] + tmp[6]*mat[1] + tmp[11]*mat[3] -	tmp[2]*mat[0] - tmp[7]*mat[1] - tmp[10]*mat[3];

	result[7]=		tmp[4]*mat[0] + tmp[9]*mat[1] + tmp[10]*mat[2] -	tmp[5]*mat[0] - tmp[8]*mat[1] - tmp[11]*mat[2];

	//calculate pairs for second 8 elements (cofactors)
	tmp[0] = mat[2]*mat[7];
	tmp[1] = mat[3]*mat[6];
	tmp[2] = mat[1]*mat[7];
	tmp[3] = mat[3]*mat[5];
	tmp[4] = mat[1]*mat[6];
	tmp[5] = mat[2]*mat[5];
	tmp[6] = mat[0]*mat[7];
	tmp[7] = mat[3]*mat[4];
	tmp[8] = mat[0]*mat[6];
	tmp[9] = mat[2]*mat[4];
	tmp[10] = mat[0]*mat[5];
	tmp[11] = mat[1]*mat[4];

	//calculate second 8 elements (cofactors)
	result[8]=		tmp[0]*mat[13] + tmp[3]*mat[14] + tmp[4]*mat[15] -	tmp[1]*mat[13] - tmp[2]*mat[14] - tmp[5]*mat[15];

	result[9]=		tmp[1]*mat[12] + tmp[6]*mat[14] + tmp[9]*mat[15] -	tmp[0]*mat[12] - tmp[7]*mat[14] - tmp[8]*mat[15];

	result[10]=		tmp[2]*mat[12] + tmp[7]*mat[13] + tmp[10]*mat[15] -	tmp[3]*mat[12] - tmp[6]*mat[13] - tmp[11]*mat[15];

	result[11]=		tmp[5]*mat[12] + tmp[8]*mat[13] + tmp[11]*mat[14] -	tmp[4]*mat[12] - tmp[9]*mat[13] - tmp[10]*mat[14];

	result[12]=		tmp[2]*mat[10] + tmp[5]*mat[11] + tmp[1]*mat[9] -	tmp[4]*mat[11] - tmp[0]*mat[9] - tmp[3]*mat[10];

	result[13]=		tmp[8]*mat[11] + tmp[0]*mat[8] + tmp[7]*mat[10] -	tmp[6]*mat[10] - tmp[9]*mat[11] - tmp[1]*mat[8];

	result[14]=		tmp[6]*mat[9] + tmp[11]*mat[11] + tmp[3]*mat[8] -	tmp[10]*mat[11] - tmp[2]*mat[8] - tmp[7]*mat[9];

	result[15]=		tmp[10]*mat[10] + tmp[4]*mat[8] + tmp[9]*mat[9] -	tmp[8]*mat[9] - tmp[11]*mat[10] - tmp[5]*mat[8];

	// calculate determinant
	det	=	 mat[0]*result[0]+mat[1]*result[1]+mat[2]*result[2]+mat[3]*result[3];

	if(det==0.0f)
	{
		double id[16]= {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
		mat = id;
	}
	else
	if(det != 1.0f)
	{
		double temp=1/det;
		result[0] *= temp;
		result[1] *= temp;
		result[2] *= temp;
		result[3] *= temp;
		result[4] *= temp;
		result[5] *= temp;
		result[6] *= temp;
		result[7] *= temp;
		result[8] *= temp;
		result[9] *= temp;
		result[10] *= temp;
		result[11] *= temp;
		result[12] *= temp;
		result[13] *= temp;
		result[14] *= temp;
		result[15] *= temp;
	}

	double out[16] = {result[ 0], result[ 4], result[ 8], result[12],
						result[ 1], result[ 5], result[ 9], result[13],
						result[ 2], result[ 6], result[10], result[14],
						result[ 3], result[ 7], result[11], result[15]};

	mat = out;
}
inline Vec3 ArbitraryRotate(Vec3 p, double theta, Vec3 r)
{
   Vec3 q(0.0,0.0,0.0);
   double costheta,sintheta;

   r = r.Normalize();
   costheta = cos(theta);
   sintheta = sin(theta);

   q.x += (float)(costheta + (1 - costheta) * r.x * r.x) * p.x;
   q.x += (float)((1 - costheta) * r.x * r.y - r.z * sintheta) * p.y;
   q.x += (float)((1 - costheta) * r.x * r.z + r.y * sintheta) * p.z;

   q.y += (float)((1 - costheta) * r.x * r.y + r.z * sintheta) * p.x;
   q.y += (float)(costheta + (1 - costheta) * r.y * r.y) * p.y;
   q.y += (float)((1 - costheta) * r.y * r.z - r.x * sintheta) * p.z;

   q.z += (float)((1 - costheta) * r.x * r.z - r.y * sintheta) * p.x;
   q.z += (float)((1 - costheta) * r.y * r.z + r.x * sintheta) * p.y;
   q.z += (float)(costheta + (1 - costheta) * r.z * r.z) * p.z;

   return(q);
}
inline void MatrixInvert4( float mat[16], float ret[16] )
{
float det;

	det = mat[0] * mat[5] * mat[10];
	det += mat[4] * mat[9] * mat[2];
	det += mat[8] * mat[1] * mat[6];
	det -= mat[8] * mat[5] * mat[2];
	det -= mat[4] * mat[1] * mat[10];
	det -= mat[0] * mat[9] * mat[6];
	det = 1.0f / det;

	ret[0] = (mat[5] * mat[10] - mat[9] * mat[6]) * det;
	ret[1] = -(mat[1] * mat[10] - mat[9] * mat[2]) * det;
	ret[2] = (mat[1] * mat[6] - mat[5] * mat[2]) * det;
	ret[3] = 0.0f;
	ret[4] = -(mat[4] * mat[10] - mat[8] * mat[6]) * det;
	ret[5] = (mat[0] * mat[10] - mat[8] * mat[2]) * det;
	ret[6] = -(mat[0] * mat[6] - mat[4] * mat[2]) * det;
	ret[7] = 0.0f;
	ret[8] = (mat[4] * mat[9] - mat[8] * mat[5]) * det;
	ret[9] = -(mat[0] * mat[9] - mat[8] * mat[1]) * det;
	ret[10] = (mat[0] * mat[5] - mat[4] * mat[1]) * det;
	ret[11] = 0.0f;
	ret[12] = -(mat[12] * ret[0] + mat[13] * ret[4] + mat[14] * ret[8]);
	ret[13] = -(mat[12] * ret[1] + mat[13] * ret[5] + mat[14] * ret[9]);
	ret[14] = -(mat[12] * ret[2] + mat[13] * ret[6] + mat[14] * ret[10]);
	ret[15] = 1.0f;
}
inline void MatrixMult(float a[16], float b[16], float* out)
{
    for (int c=0;c<4;c++)
         for (int r=0;r<4;r++)
             out[c*4+r] = a[c*4+0]*b[0+r] + a[c*4+1]*b[1*4+r] + a[c*4+2]*b[2*4+r] + a[c*4+3]*b[3*4+r];
}
//*/